/*!\file GetVectorFromControlInputsx
 * \brief retrieve vector from inputs in elements
 */

#include "./GetVectorFromControlInputsx.h"
#include "../../shared/shared.h"
#include "../../include/include.h"
#include "../../toolkits/toolkits.h"
#include "../../EnumDefinitions/EnumDefinitions.h"

void GetVectorFromControlInputsx(Vec* pvector, Elements* elements,Nodes* nodes, Vertices* vertices, Loads* loads, Materials* materials, Parameters* parameters){

	int  num_controls;
	int *control_type = NULL;
	Vec  vector=NULL;

	/*Retrieve some parameters*/
	parameters->FindParam(&num_controls,InversionNumControlParametersEnum);
	parameters->FindParam(&control_type,NULL,InversionControlParametersEnum);

	/*Allocate and populate gradient*/
	vector=NewVec(num_controls*vertices->NumberOfVertices());

	for(int i=0;i<num_controls;i++){
		for(int j=0;j<elements->Size();j++){
			Element* element=(Element*)elements->GetObjectByOffset(j);
			element->GetVectorFromControlInputs(vector,control_type[i],i);
		}
	}

	VecAssemblyBegin(vector);
	VecAssemblyEnd(vector);

	/*Assign output pointers:*/
	xfree((void**)&control_type);
	*pvector=vector;
}

void GetVectorFromControlInputsx( double** pvector, Elements* elements,Nodes* nodes, Vertices* vertices, Loads* loads, Materials* materials, Parameters* parameters){
	
	/*output: */
	double* vector=NULL;
	
	/*intermediary: */
	Vec vec_vector=NULL;

	GetVectorFromControlInputsx( &vec_vector, elements,nodes, vertices, loads, materials, parameters);
	VecToMPISerial(&vector,vec_vector);

	/*Free ressources:*/
	VecFree(&vec_vector);

	/*Assign output pointers:*/
	*pvector=vector;
}
